"use strict";
var client_1 = require("./modern/client");
var client_2 = require("./legacy/client");
function createClient(scriptURL, options) {
    if (options === void 0) { options = {}; }
    if (options.forceLegacy) {
        return new client_2.LegacyClient(scriptURL);
    }
    var useLegacy = isLegacyMode();
    /* istanbul ignore if */
    if (useLegacy) {
        console.warn('Switching to legacy mode...');
        return new client_2.LegacyClient(scriptURL);
    }
    return new client_1.ModernClient(scriptURL);
}
exports.createClient = createClient;
/* istanbul ignore next */
function isLegacyMode() {
    if (!('serviceWorker' in navigator)) {
        // tslint:disable-next-line max-line-length
        console.warn('Service worker is not supported in your browser, please check: http://caniuse.com/#feat=serviceworkers');
        return true;
    }
    if (location.protocol !== 'https:' &&
        location.hostname !== 'localhost' &&
        location.hostname !== '127.0.0.1') {
        // tslint:disable-next-line max-line-length
        console.warn('Service workers should be registered in secure pages, further information: https://github.com/w3c/ServiceWorker/blob/master/explainer.md#getting-started');
        return true;
    }
    return false;
}
//# sourceMappingURL=index.js.map