"use strict";
var tslib_1 = require("tslib");
function sendMessageRequest(target, message, timeout) {
    if (timeout === void 0) { timeout = 3 * 1e3; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, port1, port2;
        return tslib_1.__generator(this, function (_b) {
            _a = new MessageChannel(), port1 = _a.port1, port2 = _a.port2;
            return [2 /*return*/, new Promise(function (resolve, reject) {
                    var timer = isFinite(timeout) && setTimeout(function () {
                        reject(new Error("messaging timeout: " + JSON.stringify(message)));
                    }, timeout);
                    port1.onmessage = function (_a) {
                        var data = _a.data;
                        if (timer) {
                            clearTimeout(timer);
                        }
                        // avoid high transient memory usage, see
                        // https://html.spec.whatwg.org/multipage/comms.html#ports-and-garbage-collection
                        port1.close();
                        port2.close();
                        /* istanbul ignore else */
                        if (typeof data === 'object') {
                            data.request = message;
                        }
                        if (data && data.error) {
                            reject(data);
                        }
                        else {
                            resolve(data);
                        }
                    };
                    if (target === self.window) {
                        // posting message to self => legacy mode
                        // add `origin` param to `window.postMessage(message, targetOrigin, [transfer])`
                        target.postMessage(message, '*', [port2]);
                    }
                    else {
                        target.postMessage(message, [port2]);
                    }
                })];
        });
    });
}
exports.sendMessageRequest = sendMessageRequest;
//# sourceMappingURL=send-message-request.js.map